@@ -56,7 +56,6 @@
 #include "newtilesetdialog.h"
 #include "offsetmapdialog.h"
 #include "projectdock.h"
-#include "projectdocument.h"
 #include "projectmanager.h"
 #include "projectpropertiesdialog.h"
 #include "propertytypeseditor.h"
@@ -71,6 +70,8 @@
 #include "tilesetmanager.h"
 #include "tmxmapformat.h"
 #include "utils.h"
+#include "world.h"
+#include "worlddocument.h"
 #include "worldmanager.h"
 #include "zoomable.h"
 
@@ -275,6 +276,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     ActionManager::registerAction(mUi->actionLabelsForAllObjects, "LabelsForAllObjects");
     ActionManager::registerAction(mUi->actionLabelsForSelectedObjects, "LabelsForSelectedObjects");
     ActionManager::registerAction(mUi->actionLoadWorld, "LoadWorld");
+    ActionManager::registerAction(mUi->actionUnloadAllWorlds, "UnloadAllWorlds");
     ActionManager::registerAction(mUi->actionMapProperties, "MapProperties");
     ActionManager::registerAction(mUi->actionNewMap, "NewMap");
     ActionManager::registerAction(mUi->actionNewProject, "NewProject");
@@ -615,6 +617,10 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
                 mLoadedWorlds = WorldManager::instance().worlds().keys();
             });
         }
+        if (WorldManager::instance().worlds().count() >= 2) {
+            mUi->menuUnloadWorld->addSeparator();
+            mUi->menuUnloadWorld->addAction(mUi->actionUnloadAllWorlds);
+        }
     });
     connect(mUi->actionNewWorld, &QAction::triggered, this, [this] {
         Session &session = Session::current();
@@ -646,18 +652,21 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
         mUi->menuSaveWorld->clear();
 
         for (const World *world : WorldManager::instance().worlds()) {
-            if (!mDocumentManager->isWorldModified(world->fileName))
+            auto worldDocument = mDocumentManager->ensureWorldDocument(world->fileName);
+            if (!worldDocument->isModified())
                 continue;
 
-            mUi->menuSaveWorld->addAction(world->fileName, this, [this, fileName = world->fileName] {
-                QString error;
-                if (!WorldManager::instance().saveWorld(fileName, &error))
-                    QMessageBox::critical(this, tr("Error Saving World"), error);
+            mUi->menuSaveWorld->addAction(world->fileName, this, [this, worldDocument] {
+                mDocumentManager->saveDocument(worldDocument);
             });
         }
     });
+    connect(mUi->actionUnloadAllWorlds,  &QAction::triggered, this, [] {
+        WorldManager::instance().unloadAllWorlds();
+    });
     connect(mUi->menuWorld, &QMenu::aboutToShow, this, [this] {
         mUi->menuUnloadWorld->setEnabled(!WorldManager::instance().worlds().isEmpty());
+        mUi->actionUnloadAllWorlds->setEnabled(!WorldManager::instance().worlds().isEmpty());
         mUi->menuSaveWorld->setEnabled(DocumentManager::instance()->isAnyWorldModified());
     });
     connect(mUi->actionResizeMap, &QAction::triggered, this, &MainWindow::resizeMap);
@@ -1204,14 +1213,12 @@ void MainWindow::saveAll()
     }
 
     for (const World *world : WorldManager::instance().worlds()) {
-        if (!mDocumentManager->isWorldModified(world->fileName))
+        auto worldDocument = mDocumentManager->ensureWorldDocument(world->fileName);
+        if (!worldDocument->isModified())
             continue;
 
-        QString error;
-        if (!WorldManager::instance().saveWorld(world->fileName, &error)) {
-            QMessageBox::critical(this, tr("Error Saving World"), error);
+        if (!mDocumentManager->saveDocument(worldDocument))
             return;
-        }
     }
 }
 
@@ -1255,7 +1262,8 @@ bool MainWindow::confirmAllSave()
 
 bool MainWindow::confirmSaveWorld(const QString &fileName)
 {
-    if (!mDocumentManager->isWorldModified(fileName))
+    auto worldDocument = mDocumentManager->ensureWorldDocument(fileName);
+    if (!worldDocument->isModified())
         return true;
 
     int ret = QMessageBox::warning(
@@ -1264,15 +1272,8 @@ bool MainWindow::confirmSaveWorld(const QString &fileName)
             QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
 
     switch (ret) {
-    case QMessageBox::Save: {
-        QString error;
-        if (!WorldManager::instance().saveWorld(fileName, &error)) {
-            QMessageBox::critical(window(), tr("Error Saving World"), error);
-            return false;
-        }
-
-        return true;
-    }
+    case QMessageBox::Save:
+        return mDocumentManager->saveDocument(worldDocument, fileName);
     case QMessageBox::Discard:
         return true;
     case QMessageBox::Cancel:
@@ -1597,7 +1598,7 @@ void MainWindow::openProjectExtensionsPopup()
 
     auto label = new QLabel;
     label->setTextFormat(Qt::RichText);
-    label->setText(tr("The current project contains <a href=\"https://doc.mapeditor.org/en/stable/reference/scripting/\">scripted extensions</a>.<br><i>Make sure you trust those extensions before enabling them!</i>"));
+    label->setText(tr("The current project contains <a href=\"https://doc.mapeditor.org/en/stable/manual/scripting/\">scripted extensions</a>.<br><i>Make sure you trust those extensions before enabling them!</i>"));
     label->setOpenExternalLinks(true);
 
     auto enableButton = new QPushButton(tr("&Enable Extensions"));
@@ -2150,7 +2151,7 @@ void MainWindow::updateActions()
     mUi->actionExportAsImage->setEnabled(mapDocument);
     mUi->actionExport->setEnabled(mapDocument || tilesetDocument);
     mUi->actionExportAs->setEnabled(mapDocument || tilesetDocument);
-    mUi->actionReload->setEnabled(mapDocument || (tilesetDocument && tilesetDocument->canReload()));
+    mUi->actionReload->setEnabled(document && document->canReload());
     mUi->actionClose->setEnabled(document);
     mUi->actionCloseAll->setEnabled(document);
 
@@ -2322,9 +2323,8 @@ void MainWindow::retranslateUi()
 void MainWindow::exportMapAs(MapDocument *mapDocument)
 {
     SessionOption<QString> lastUsedExportFilter { "map.lastUsedExportFilter" };
-    QString fileName = mapDocument->fileName();
     QString selectedFilter = lastUsedExportFilter;
-    auto exportDetails = chooseExportDetails<MapFormat>(fileName,
+    auto exportDetails = chooseExportDetails<MapFormat>(mapDocument->fileName(),
                                                         mapDocument->lastExportFileName(),
                                                         selectedFilter,
                                                         this);
