@@ -32,7 +32,6 @@
 #include "createtextobjecttool.h"
 #include "createtileobjecttool.h"
 #include "documentmanager.h"
-#include "editablemanager.h"
 #include "editablemap.h"
 #include "editablewangset.h"
 #include "editpolygontool.h"
@@ -51,7 +50,7 @@
 #include "objectreferencetool.h"
 #include "objectsdock.h"
 #include "objectselectiontool.h"
-#include "objecttemplate.h"
+#include "objecttemplate.h"         // used when compiling against Qt 5
 #include "preferences.h"
 #include "propertiesdock.h"
 #include "reversingproxymodel.h"
@@ -74,10 +73,9 @@
 #include "undodock.h"
 #include "wangbrush.h"
 #include "wangdock.h"
-#include "wangset.h"
 #include "zoomable.h"
-#include "worldmovemaptool.h"
 #include "worldmanager.h"
+#include "worldmovemaptool.h"
 
 #include <QComboBox>
 #include <QDialogButtonBox>
@@ -208,7 +206,7 @@ MapEditor::MapEditor(QObject *parent)
     mToolsToolBar->addAction(mToolManager->registerTool(new LayerOffsetTool(this)));
     mToolsToolBar->addSeparator();  // todo: hide when there are no tool extensions
 
-    const auto tools = PluginManager::instance()->objects<AbstractTool>();
+    const auto tools = PluginManager::objects<AbstractTool>();
     for (auto tool : tools)
         mToolsToolBar->addAction(mToolManager->registerTool(tool));
 
@@ -1064,8 +1062,7 @@ void MapEditor::setCurrentBrush(EditableMap *editableMap)
 
 EditableWangSet *MapEditor::currentWangSet() const
 {
-    auto currentWangSet = mWangDock->currentWangSet();
-    return EditableManager::instance().editableWangSet(currentWangSet);
+    return EditableWangSet::get(mWangDock->currentWangSet());
 }
 
 int MapEditor::currentWangColorIndex() const
