@@ -38,6 +38,7 @@
 #include "tilelayer.h"
 #include "tilelayeritem.h"
 #include "tileselectionitem.h"
+#include "world.h"
 #include "worldmanager.h"
 #include "zoomable.h"
 
@@ -357,6 +358,21 @@ void MapItem::repaintRegion(const QRegion &region, TileLayer *tileLayer)
 void MapItem::documentChanged(const ChangeEvent &change)
 {
     switch (change.type) {
+    case ChangeEvent::DocumentAboutToReload:
+        for (Layer *layer : mMapDocument->map()->layers())
+            deleteLayerItems(layer);
+        break;
+    case ChangeEvent::DocumentReloaded: {
+        // The renderer has been re-created
+        auto lineWidth = Preferences::instance()->objectLineWidth();
+        mapDocument()->renderer()->setObjectLineWidth(lineWidth);
+
+        createLayerItems(mMapDocument->map()->layers());
+
+        updateBoundingRect();
+        updateLayerPositions();
+        break;
+    }
     case ChangeEvent::ObjectsChanged: {
         auto &objectsChange = static_cast<const ObjectsChangeEvent&>(change);
         if (!objectsChange.objects.isEmpty() && (objectsChange.properties & ObjectsChangeEvent::ClassProperty)) {
@@ -442,7 +458,7 @@ void MapItem::mapChanged()
     updateBoundingRect();
 
     // When this map is part of a world, update that map's rect when necessary
-    const QString mapFileName = mapDocument()->fileName();
+    const QString &mapFileName = mapDocument()->fileName();
     if (const World *world = WorldManager::instance().worldForMap(mapFileName)) {
         if (world->canBeModified()) {
             const QRect currentRectInWorld = world->mapRect(mapFileName);
