@@ -38,6 +38,9 @@
 #include "scriptmanager.h"
 #include "tilesetdocument.h"
 #include "tileseteditor.h"
+#include "world.h"
+#include "worlddocument.h"
+#include "worldmanager.h"
 
 #include <QAction>
 #include <QCoreApplication>
@@ -59,10 +62,13 @@ ScriptModule::ScriptModule(QObject *parent)
     if (auto documentManager = DocumentManager::maybeInstance()) {
         connect(documentManager, &DocumentManager::documentCreated, this, &ScriptModule::documentCreated);
         connect(documentManager, &DocumentManager::documentOpened, this, &ScriptModule::documentOpened);
+        connect(documentManager, &DocumentManager::documentReloaded, this, &ScriptModule::documentReloaded);
         connect(documentManager, &DocumentManager::documentAboutToBeSaved, this, &ScriptModule::documentAboutToBeSaved);
         connect(documentManager, &DocumentManager::documentSaved, this, &ScriptModule::documentSaved);
         connect(documentManager, &DocumentManager::documentAboutToClose, this, &ScriptModule::documentAboutToClose);
         connect(documentManager, &DocumentManager::currentDocumentChanged, this, &ScriptModule::currentDocumentChanged);
+
+        connect(&WorldManager::instance(), &WorldManager::worldsChanged, this, &ScriptModule::worldsChanged);
     }
 }
 
@@ -234,6 +240,20 @@ MapEditor *ScriptModule::mapEditor() const
     return nullptr;
 }
 
+QColor ScriptModule::color(const QString &name) const
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 4, 0)
+    return QColor::isValidColor(name) ? QColor(name) : QColor();
+#else
+    return QColor::fromString(name);
+#endif
+}
+
+QColor ScriptModule::color(float r, float g, float b, float a) const
+{
+    return QColor::fromRgbF(r, g, b, a);
+}
+
 FilePath ScriptModule::filePath(const QUrl &path) const
 {
     return { path };
@@ -244,7 +264,7 @@ ObjectRef ScriptModule::objectRef(int id) const
     return { id };
 }
 
-QVariant ScriptModule::propertyValue(const QString &typeName, const QVariant &value) const
+QVariant ScriptModule::propertyValue(const QString &typeName, const QJSValue &value) const
 {
     auto type = Object::propertyTypes().findPropertyValueType(typeName);
     if (!type) {
@@ -252,16 +272,22 @@ QVariant ScriptModule::propertyValue(const QString &typeName, const QVariant &va
         return {};
     }
 
+    const QVariant var = value.toVariant();
+
     switch (type->type) {
     case PropertyType::PT_Invalid:
     case PropertyType::PT_Class:
+        if (var.userType() != QVariant::Map) {
+            ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Expected object to initialize class value"));
+            return {};
+        }
         break;
     case PropertyType::PT_Enum:
         // Call toPropertyValue to support using strings to create a value
-        return type->toPropertyValue(value, ExportContext());
+        return type->toPropertyValue(var, ExportContext());
     }
 
-    return type->wrap(value);
+    return type->wrap(var);
 }
 
 bool ScriptModule::versionLessThan(const QString &a, const QString &b)
@@ -671,6 +697,11 @@ void ScriptModule::documentOpened(Document *document)
     emit assetOpened(document->editable());
 }
 
+void ScriptModule::documentReloaded(Document *document)
+{
+    emit assetReloaded(document->editable());
+}
+
 void ScriptModule::documentAboutToBeSaved(Document *document)
 {
     emit assetAboutToBeSaved(document->editable());
@@ -691,6 +722,37 @@ void ScriptModule::currentDocumentChanged(Document *document)
     emit activeAssetChanged(document ? document->editable() : nullptr);
 }
 
+QList<QObject *> ScriptModule::worlds() const
+{
+    QList<QObject*> worlds;
+
+    auto documentManager = DocumentManager::maybeInstance();
+    if (!documentManager)
+        return worlds;
+
+    for (const World *world : WorldManager::instance().worlds()) {
+        WorldDocument *worldDocument = documentManager->ensureWorldDocument(world->fileName);
+        worlds.append(worldDocument->editable());
+    }
+
+    return worlds;
+}
+
+void ScriptModule::loadWorld(const QString &fileName) const
+{
+    WorldManager::instance().loadWorld(fileName);
+}
+
+void ScriptModule::unloadWorld(const QString &fileName) const
+{
+    WorldManager::instance().unloadWorld(fileName);
+}
+
+void ScriptModule::unloadAllWorlds() const
+{
+    WorldManager::instance().unloadAllWorlds();
+}
+
 } // namespace Tiled
 
 #include "moc_scriptmodule.cpp"
