@@ -27,7 +27,6 @@
 #include "addremovetileset.h"
 #include "containerhelpers.h"
 #include "documentmanager.h"
-#include "editablemanager.h"
 #include "editabletile.h"
 #include "erasetiles.h"
 #include "map.h"
@@ -938,9 +937,8 @@ void TilesetDock::tabContextMenuRequested(const QPoint &pos)
     QMenu menu;
 
     auto tilesetDocument = mTilesetDocuments.at(index);
-    const QString fileName = tilesetDocument->fileName();
 
-    Utils::addFileManagerActions(menu, fileName);
+    Utils::addFileManagerActions(menu, tilesetDocument->fileName());
 
     menu.addSeparator();
     menu.addAction(mEditTileset->icon(), mEditTileset->text(), this, [tileset = tilesetDocument->tileset()] {
@@ -1033,10 +1031,9 @@ QList<QObject *> TilesetDock::selectedTiles() const
     EditableTileset *editableTileset = currentEditableTileset();
 
     const TilesetModel *model = view->tilesetModel();
-    auto &editableManager = EditableManager::instance();
     for (const QModelIndex &index : indexes)
         if (Tile *tile = model->tileAt(index))
-            result.append(editableManager.editableTile(editableTileset, tile));
+            result.append(EditableTile::get(editableTileset, tile));
 
     return result;
 }
